@@ -766,7 +766,13 @@ class AnnotationParser(object):
         comment_blocks = {}
 
         for comment in comments:
-            comment_block = self.parse_comment_block(comment)
+            try:
+                comment_block = self.parse_comment_block(comment)
+            except Exception:
+                message.warn('unrecoverable parse error, please file a GObject-Introspection '
+                             'bug report including the complete comment block at the '
+                             'indicated location.', message.Position(comment[1], comment[2]))
+                continue
 
             if comment_block is not None:
                 # Note: previous versions of this parser did not check
@@ -1119,20 +1125,23 @@ class AnnotationParser(object):
         ########################################################################
         # Finished parsing this comment block.
         ########################################################################
-        # We have picked up a couple of \n characters that where not
-        # intended. Strip those.
-        if comment_block.comment:
-            comment_block.comment = comment_block.comment.strip()
+        if comment_block:
+            # We have picked up a couple of \n characters that where not
+            # intended. Strip those.
+            if comment_block.comment:
+                comment_block.comment = comment_block.comment.strip()
 
-        for tag in comment_block.tags.values():
-            self._clean_comment_block_part(tag)
+            for tag in comment_block.tags.values():
+                self._clean_comment_block_part(tag)
 
-        for param in comment_block.params.values():
-            self._clean_comment_block_part(param)
+            for param in comment_block.params.values():
+                self._clean_comment_block_part(param)
 
-        # Validate and store block.
-        comment_block.validate()
-        return comment_block
+            # Validate and store block.
+            comment_block.validate()
+            return comment_block
+        else:
+            return None
 
     def _clean_comment_block_part(self, part):
         if part.comment:
