@@ -39,7 +39,7 @@ def doc_main(args):
     if not options.output:
         raise SystemExit("missing output parameter")
     if not os.path.isdir(options.output):
-        raise SystemExit("wrong output parameter: %s", options.output)
+        raise SystemExit("wrong output parameter: %s" % (options.output, ))
 
     if len(args) < 2:
         raise SystemExit("Need an input GIR filename")
