@@ -27,14 +27,17 @@ import tempfile
 
 from xml.sax import saxutils
 from mako.template import Template
+from mako.lookup import TemplateLookup
 
 from . import ast
 from .utils import to_underscores
 
-def make_page_id(namespace, node):
+def make_page_id(node):
     if isinstance(node, ast.Namespace):
         return 'index'
-    elif isinstance(node, (ast.Class, ast.Interface)):
+
+    namespace = node.namespace
+    if isinstance(node, (ast.Class, ast.Interface)):
         return '%s.%s' % (namespace.name, node.name)
     elif isinstance(node, ast.Record):
         return '%s.%s' % (namespace.name, node.name)
@@ -148,6 +151,7 @@ class DocstringScanner(TemplatedScanner):
             ('signal', r'#<<type_name:alpha>>::(<<signal_name:alpha_dash>>)'),
             ('type_name', r'#(<<type_name:alpha>>)'),
             ('fundamental', r'%(<<fundamental:alpha>>)'),
+            ('parameter', r'@<<param_name:alpha>>'),
             ('function_call', r'<<symbol_name:alpha>>\(\)'),
         ]
 
@@ -161,20 +165,17 @@ class MallardFormatter(object):
     def escape(self, text):
         return saxutils.escape(text)
 
-    def format(self, doc):
+    def format(self, node, doc):
         if doc is None:
             return ''
 
         result = ''
         for para in doc.split('\n\n'):
             result += '<p>'
-            result += self.format_inline(para)
+            result += self.format_inline(node, para)
             result += '</p>'
         return result
 
-    def _process_other(self, namespace, match, props):
-        return self.escape(match)
-
     def _resolve_type(self, ident):
         try:
             matches = self._transformer.split_ctype_namespaces(ident)
@@ -203,53 +204,59 @@ class MallardFormatter(object):
                 return item
         raise KeyError("Could not find %s" % (name, ))
 
-    def _process_property(self, namespace, match, props):
+    def _process_other(self, node, match, props):
+        return self.escape(match)
+
+    def _process_property(self, node, match, props):
         type_node = self._resolve_type(props['type_name'])
         if type_node is None:
             return match
 
         try:
-            node = self._find_thing(type_node.properties, props['property_name'])
+            prop = self._find_thing(type_node.properties, props['property_name'])
         except (AttributeError, KeyError), e:
             return match
 
-        xref_name = "%s.%s:%s" % (namespace.name, type_node.name, node.name)
-        return '<link xref="%s">%s</link>' % (make_page_id(namespace, node), xref_name)
+        return self.format_xref(prop)
 
-    def _process_signal(self, namespace, match, props):
+    def _process_signal(self, node, match, props):
         type_node = self._resolve_type(props['type_name'])
         if type_node is None:
             return match
 
         try:
-            node = self._find_thing(type_node.signals, props['signal_name'])
+            signal = self._find_thing(type_node.signals, props['signal_name'])
         except (AttributeError, KeyError), e:
             return match
 
-        xref_name = "%s.%s::%s" % (node.namespace.name, type_node.name, node.name)
-        return '<link xref="%s">%s</link>' % (make_page_id(node.namespace, node), xref_name)
+        return self.format_xref(signal)
 
-    def _process_type_name(self, namespace, match, props):
-        node = self._resolve_type(props['type_name'])
-        if node is None:
-            return match
-        xref_name = "%s.%s" % (node.namespace.name, node.name)
-        return '<link xref="%s">%s</link>' % (make_page_id(node.namespace, node), xref_name)
-
-    def _process_function_call(self, namespace, match, props):
-        node = self._resolve_symbol(props['symbol_name'])
-        if node is None:
+    def _process_type_name(self, node, match, props):
+        type_ = self._resolve_type(props['type_name'])
+        if type_ is None:
             return match
 
-        return '<link xref="%s">%s</link>' % (make_page_id(node.namespace, node),
-                                              self.format_function_name(node))
+        return self.format_xref(type_)
 
-    def _process_fundamental(self, namespace, match, props):
+    def _process_fundamental(self, node, match, props):
         return self.fundamentals.get(props['fundamental'], match)
 
-    def _process_token(self, tok):
-        namespace = self._transformer.namespace
+    def _process_parameter(self, node, match, props):
+        try:
+            parameter = node.get_parameter(props['param_name'])
+        except (AttributeError, ValueError), e:
+            return match
 
+        return '<code>%s</code>' % (self.format_parameter_name(node, parameter), )
+
+    def _process_function_call(self, node, match, props):
+        func = self._resolve_symbol(props['symbol_name'])
+        if func is None:
+            return match
+
+        return self.format_xref(func)
+
+    def _process_token(self, node, tok):
         kind, match, props = tok
 
         dispatch = {
@@ -257,23 +264,45 @@ class MallardFormatter(object):
             'property': self._process_property,
             'signal': self._process_signal,
             'type_name': self._process_type_name,
-            'function_call': self._process_function_call,
             'fundamental': self._process_fundamental,
+            'parameter': self._process_parameter,
+            'function_call': self._process_function_call,
         }
 
-        return dispatch[kind](namespace, match, props)
+        return dispatch[kind](node, match, props)
 
-    def format_inline(self, para):
+    def format_inline(self, node, para):
         tokens = self._scanner.scan(para)
-        words = [self._process_token(tok) for tok in tokens]
+        words = [self._process_token(node, tok) for tok in tokens]
         return ''.join(words)
 
+    def format_parameter_name(self, node, parameter):
+        return parameter.argname
+
     def format_function_name(self, func):
         raise NotImplementedError
 
     def format_type(self, type_):
         raise NotImplementedError
 
+    def format_page_name(self, node):
+        namespace = node.namespace
+        if isinstance(node, ast.Namespace):
+            return 'Index'
+        elif isinstance(node, ast.Function):
+            return self.format_function_name(node)
+        elif isinstance(node, ast.Property) and node.parent is not None:
+            return '%s.%s:%s' % (namespace.name, node.parent.name, node.name)
+        elif isinstance(node, ast.Signal) and node.parent is not None:
+            return '%s.%s::%s' % (namespace.name, node.parent.name, node.name)
+        elif isinstance(node, ast.VFunction) and node.parent is not None:
+            return '%s.%s::%s' % (namespace.name, node.parent.name, node.name)
+        else:
+            return make_page_id(node)
+
+    def format_xref(self, node):
+        return '<link xref="%s">%s</link>' % (make_page_id(node), self.format_page_name(node))
+
     def format_property_flags(self, property_, construct_only=False):
         flags = []
         if property_.readable and not construct_only:
@@ -329,6 +358,46 @@ class MallardFormatterPython(MallardFormatter):
         "NULL": "None",
     }
 
+    def is_method(self, node):
+        if getattr(node, "is_method", False):
+            return True
+
+        if isinstance(node, (ast.VFunction)):
+            return True
+
+        return False
+
+    def format_parameter_name(self, node, parameter):
+        # Force "self" for the first parameter of a method
+        if self.is_method(node) and parameter is node.parameters[0]:
+            return "self"
+        else:
+            return parameter.argname
+
+    def format_fundamental_type(self, name):
+        fundamental_types = {
+            "utf8": "unicode",
+            "gunichar": "unicode",
+            "gchar": "str",
+            "guchar": "str",
+            "gboolean": "bool",
+            "gint": "int",
+            "guint": "int",
+            "glong": "int",
+            "gulong": "int",
+            "gint64": "int",
+            "guint64": "int",
+            "gfloat": "float",
+            "gdouble": "float",
+            "gchararray": "str",
+            "GParam": "GLib.Param",
+            "PyObject": "object",
+            "GStrv": "[str]",
+            "GVariant": "GLib.Variant",
+            }
+
+        return fundamental_types.get(name, name)
+
     def format_type(self, type_):
         if isinstance(type_, ast.Array):
             return '[' + self.format_type(type_.element_type) + ']'
@@ -338,7 +407,7 @@ class MallardFormatterPython(MallardFormatter):
         elif type_.target_giname is not None:
             return type_.target_giname
         else:
-            return type_.target_fundamental
+            return self.format_fundamental_type(type_.target_fundamental)
 
     def format_function_name(self, func):
         if func.parent is not None:
@@ -363,6 +432,21 @@ class MallardWriter(object):
         self._formatter = formatter_class(self._transformer)
         self._language = self._formatter.language
 
+        self._lookup = self._get_template_lookup()
+
+    def _get_template_lookup(self):
+        if 'UNINSTALLED_INTROSPECTION_SRCDIR' in os.environ:
+            top_srcdir = os.environ['UNINSTALLED_INTROSPECTION_SRCDIR']
+            srcdir = os.path.join(top_srcdir, 'giscanner')
+        else:
+            srcdir = os.path.dirname(__file__)
+
+        template_dir = os.path.join(srcdir, 'doctemplates', self._language)
+
+        return TemplateLookup(directories=[template_dir],
+                              module_directory=tempfile.gettempdir(),
+                              output_encoding='utf-8')
+
     def write(self, output):
         nodes = [self._transformer.namespace]
         for node in self._transformer.namespace.itervalues():
@@ -389,19 +473,10 @@ class MallardWriter(object):
     def _render_node(self, node, output):
         namespace = self._transformer.namespace
 
-        if 'UNINSTALLED_INTROSPECTION_SRCDIR' in os.environ:
-            top_srcdir = os.environ['UNINSTALLED_INTROSPECTION_SRCDIR']
-            template_dir = os.path.join(top_srcdir, 'giscanner')
-        else:
-            template_dir = os.path.dirname(__file__)
-
         template_name = make_template_name(node, self._language)
-        page_id = make_page_id(namespace, node)
+        page_id = make_page_id(node)
 
-        file_name = os.path.join(template_dir, template_name)
-        file_name = os.path.abspath(file_name)
-        template = Template(filename=file_name, output_encoding='utf-8',
-                            module_directory=tempfile.gettempdir())
+        template = self._lookup.get_template(template_name)
         result = template.render(namespace=namespace,
                                  node=node,
                                  page_id=page_id,
