@@ -389,6 +389,7 @@ void gi_marshalling_tests_array_in_len_zero_terminated (const gint *ints, gint l
 void gi_marshalling_tests_array_string_in (const gchar **strings, gint length);
 void gi_marshalling_tests_array_uint8_in (const guint8 *chars, gint length);
 void gi_marshalling_tests_array_struct_in (GIMarshallingTestsBoxedStruct **structs, gint length);
+void gi_marshalling_tests_array_struct_value_in (GIMarshallingTestsBoxedStruct *structs, gint length);
 void gi_marshalling_tests_array_struct_take_in (GIMarshallingTestsBoxedStruct **structs, gint length);
 void gi_marshalling_tests_array_simple_struct_in (GIMarshallingTestsSimpleStruct *structs, gint length);
 void gi_marshalling_tests_multi_array_key_value_in (gint length, const gchar **keys, const GValue *values);
@@ -572,7 +573,7 @@ GClosure *gi_marshalling_tests_gclosure_return (void);
 /**
  * GIMarshallingTestsCallbackReturnValueOnly:
  */
-typedef glong (* GIMarshallingTestsCallbackReturnValueOnly) ();
+typedef glong (* GIMarshallingTestsCallbackReturnValueOnly) (void);
 
 glong gi_marshalling_tests_callback_return_value_only (GIMarshallingTestsCallbackReturnValueOnly callback);
 
@@ -655,6 +656,7 @@ void gi_marshalling_tests_pointer_struct_inv (GIMarshallingTestsPointerStruct *s
 
 struct _GIMarshallingTestsBoxedStruct {
     glong long_;
+    gchar *string_;
     GStrv g_strv;
 };
 
@@ -776,6 +778,12 @@ struct _GIMarshallingTestsObjectClass
      */
     void  (* vfunc_caller_allocated_out_parameter) (GIMarshallingTestsObject *self, GValue *a);
 
+    /**
+     * GIMarshallingTestsObjectClass::vfunc_array_out_parameter:
+     * @a: (out) (array zero-terminated=1):
+     */
+    void  (* vfunc_array_out_parameter) (GIMarshallingTestsObject *self, gfloat **a);
+
     /**
      * GIMarshallingTestsObjectClass::vfunc_return_value_and_one_out_parameter:
      * @a: (out):
@@ -827,6 +835,7 @@ glong gi_marshalling_tests_object_vfunc_return_value_only (GIMarshallingTestsObj
 void gi_marshalling_tests_object_vfunc_one_out_parameter (GIMarshallingTestsObject *self, gfloat *a);
 void gi_marshalling_tests_object_vfunc_multiple_out_parameters (GIMarshallingTestsObject *self, gfloat *a, gfloat *b);
 void gi_marshalling_tests_object_vfunc_caller_allocated_out_parameter (GIMarshallingTestsObject *self, GValue *a);
+void gi_marshalling_tests_object_vfunc_array_out_parameter (GIMarshallingTestsObject *self, gfloat **a);
 glong gi_marshalling_tests_object_vfunc_return_value_and_one_out_parameter (GIMarshallingTestsObject *self, glong *a);
 glong gi_marshalling_tests_object_vfunc_return_value_and_multiple_out_parameters (GIMarshallingTestsObject *self, glong *a, glong *b);
 gboolean gi_marshalling_tests_object_vfunc_meth_with_error (GIMarshallingTestsObject *object, gint x, GError **error);
@@ -960,7 +969,7 @@ typedef struct _GIMarshallingTestsInterface3Iface GIMarshallingTestsInterface3If
 
 struct _GIMarshallingTestsInterface3Iface {
     GTypeInterface base_iface;
- 
+
     /**
      * GIMarshallingTestsInterface3::test_variant_array_in:
      * @in: (in) (array length=n_in):
@@ -994,6 +1003,7 @@ void gi_marshalling_tests_gerror_out_transfer_none(GError **err, const gchar **d
 GError *gi_marshalling_tests_gerror_return(void);
 
 /* GParamSpec */
+void gi_marshalling_tests_param_spec_in_bool(const GParamSpec *param);
 GParamSpec *gi_marshalling_tests_param_spec_return (void);
 void gi_marshalling_tests_param_spec_out(GParamSpec **param);
 
@@ -1072,7 +1082,8 @@ struct _GIMarshallingTestsPropertiesObject {
     gfloat some_float;
     gdouble some_double;
     gchar **some_strv;
-    GIMarshallingTestsBoxedStruct* some_boxed_struct; 
+    GIMarshallingTestsBoxedStruct* some_boxed_struct;
+    GList* some_boxed_glist;
     GVariant *some_variant;
 };
 
